// Testing program for the Gene class
// John Dunkelberg

#include <iostream.h>
#include <fstream.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/resource.h>

#include "config.H"
#include "population.H"

extern "C"
{
  void srandom(int);
  long random(void);
  void srand48(long);
  double drand48(void);
  char *sprintf(const char *fmt, ... );
  char *strdup(char *);

  struct rusage usage_info;
}

main(int argc, char **argv)
{
  Pop Alpha;

  long pop_size;
  int order;
  int generation;
  char *mesh_fname;
  char *output_fname;

  double best_fit, avg_fit;

  mesh_fname = new char[MAXLINE];
  output_fname = new char[MAXLINE];

  if (argc > 1)
    {
      mesh_fname = strdup(argv[1]);
      order = atoi(argv[2]);
    }
  else
    {
      cout << "\nMesh filename? ";
      cin >> mesh_fname;
      cout << "\nOrder ?       ";
      cin >> order;
    }

  srandom(random_seed);
  srand48((long) random_seed);

  fstream in_fp(mesh_fname,ios::in);

  sprintf(output_fname,"res.%s.%d.%d.%d.%d.%d.%d",mesh_fname,order,
	  init_select_gen,cut_every_other_gen,cut_pop_gen,prim_gen,max_gen);
  fstream out_fp(output_fname,ios::out);

  cout << Alpha.Init_File(in_fp) << endl;
  pop_size = Alpha.Initialize(order);
  cout << "Initialization Done, pop_size = " << pop_size << endl;
  Alpha.Init_Tplate();
  //  Alpha.Show();

  for(generation=0;(generation < init_select_gen);generation++)
    {
      cout << "\nGeneration : " << generation << endl;
      pop_size = Alpha.Init_Select(pop_size);
      avg_fit = Alpha.Statistics();
      best_fit = Alpha.Get_Best_Fit();
      cout << "Average Fitness: " << avg_fit << endl;
      cout << "Best Fitness   : " << best_fit << endl;
      out_fp << generation << " " << avg_fit << " " << best_fit << endl;
      cout << "Best Chromosone: \n";
      Alpha.Show_Best(11);
      // Alpha.Show();
    }
  for(;(generation < cut_every_other_gen);generation++)
    {
      cout << "\nGeneration : " << generation << endl;
      if (generation %2)
	pop_size = Alpha.Select(pop_size/2);
      else
	pop_size = Alpha.Select(pop_size);
      avg_fit = Alpha.Statistics();
      best_fit = Alpha.Get_Best_Fit();
      cout << "Average Fitness: " << avg_fit << endl;
      cout << "Best Fitness   : " << best_fit << endl;
      out_fp << generation << " " << avg_fit << " " << best_fit << endl;
      cout << "Best Chromosone: \n";
      Alpha.Show_Best(11);
      // Alpha.Show();
    }
  for(;(generation < cut_pop_gen);generation++)
    {
      cout << "\nGeneration : " << generation << endl;
      pop_size = Alpha.Select(pop_size/2);
      avg_fit = Alpha.Statistics();
      best_fit = Alpha.Get_Best_Fit();
      cout << "Average Fitness: " << avg_fit << endl;
      cout << "Best Fitness   : " << best_fit << endl;
      out_fp << generation << " " << avg_fit << " " << best_fit << endl;
      cout << "Best Chromosone: \n";
      Alpha.Show_Best(11);
      // Alpha.Show();
    }
  for(;(generation < prim_gen);generation++)
    {
      cout << "\nGeneration : " << generation << endl;
      pop_size = Alpha.Select(pop_size);
      avg_fit = Alpha.Statistics();
      best_fit = Alpha.Get_Best_Fit();
      cout << "Average Fitness: " << avg_fit << endl;
      cout << "Best Fitness   : " << best_fit << endl;
      out_fp << generation << " " << avg_fit << " " << best_fit << endl;
      cout << "Best Chromosone: \n";
      Alpha.Show_Best(11);
      // Alpha.Show();
    }
  for(;(generation < max_gen);generation++)
    {
      cout << "\nGeneration : " << generation << endl;
      pop_size = Alpha.Juxta(pop_size);
      avg_fit = Alpha.Statistics();
      best_fit = Alpha.Get_Best_Fit();
      cout << "Average Fitness: " << avg_fit << endl;
      cout << "Best Fitness   : " << best_fit << endl;
      out_fp << generation << " " << avg_fit << " " << best_fit << endl;
      cout << "Best Chromosone: \n";
      Alpha.Show_Best(11);
      // Alpha.Show();      
    }

  Alpha.Show();

  getrusage(RUSAGE_SELF,&usage_info);
  cout << "\n CPU time, user  : "<< usage_info.ru_utime.tv_sec<< " sec ";
  cout << usage_info.ru_utime.tv_usec << " usec";	 
  cout << "\n CPU time, system: "<< usage_info.ru_stime.tv_sec <<" sec ";
  cout <<  usage_info.ru_stime.tv_usec <<"usec";
  cout << "\n Involuntary Context Switches: " << usage_info.ru_nivcsw;
  cout << "\n Voluntary Context Switches  : " << usage_info.ru_nvcsw;
  cout << "\n Major Page Faults : " << usage_info.ru_majflt;
  cout << "\n Minor Page Faults : " << usage_info.ru_minflt;
  cout << endl;
  
  out_fp << "\n CPU time, user  : "<< usage_info.ru_utime.tv_sec<< " sec ";
  out_fp << usage_info.ru_utime.tv_usec << " usec";	 
  out_fp << "\n CPU time, system: "<< usage_info.ru_stime.tv_sec <<" sec ";
  out_fp <<  usage_info.ru_stime.tv_usec <<"usec";
  out_fp << "\n Involuntary Context Switches: " << usage_info.ru_nivcsw;
  out_fp << "\n Voluntary Context Switches  : " << usage_info.ru_nvcsw;
  out_fp << "\n Major Page Faults : " << usage_info.ru_majflt;
  out_fp << "\n Minor Page Faults : " << usage_info.ru_minflt;
  out_fp << endl;

  cout << "End of Simulation\n";
  out_fp.close();

}
