/*
 * Problem Class Definition File
 * holds mesh info, also holds other problem information
 * as a method, this also figures the fitness of a chromosone (as a friend)
 */

extern int DEBUG;

class Problem
{
public:
  // Mesh information
  int size;           // size of the problem mesh (# nodes)
  int connect;        // max connectivity of the mesh
  int *edge;          // array[][] of edges masquerading as a 1-D

  // Machine Definition Information (assumes toroidal wrap SIMD)
  int mach_side;      // number of nodes on a side 
  int mach_size;      // number of nodes total

  // Functions
  Problem(void);
  ~Problem(void)
    { delete edge; }
  Initialize(fstream);
  
  // General Info Functions
  inline int Get_Size(void)
    { return size; }        // get the size of the mesh (nodes)
  inline int Get_Connect(void)
    { return connect; }     // get the connectivity of the mesh
  inline int Get_Mach_Side(void)
    { return mach_side; }   // get the machine side length
  inline int Get_Mach_Size(void)
    { return mach_size; }   // get the machine size
  inline int Get_Edge(int x,int y)
    { return edge[(x*connect) + y]; }  // get a particular edge
  void Show();
  void Show(int);
};

// Constructor for Problem
Problem::Problem(void)
{
  size = connect = mach_side = mach_size = 0;
  edge = (int *) NULL;
}

// Initialization of problem via file desription
Problem::Initialize(fstream fp)
{
  int node,side;
  char *com_buf,c;

  com_buf = new char[MAXLINE];

  // problem initializer
  while (fp.peek() == '#')
    {
      fp.get(com_buf,MAXLINE,'\n');
      if (DEBUG > 50)
	cout << "Disregard: " << com_buf << endl;
      fp.get(c);
    }
  fp >> mach_side;
  fp.get(c);
  mach_size = mach_side * mach_side;
  while (fp.peek() == '#')
    {
      fp.get(com_buf,MAXLINE,'\n');
      if (DEBUG > 50)
	cout << "Disregard: " << com_buf << endl;
      fp.get(c);
    }
  fp >> size;
  fp.get(c);
  while (fp.peek() == '#')
    {
      fp.get(com_buf,MAXLINE,'\n');
      if (DEBUG > 50)
	cout << "Disregard: " << com_buf << endl;
      fp.get(c);
    }
  fp >> connect;
  fp.get(c);

  if (DEBUG > 20)
    cout << "ms:"<<mach_side<<" s:"<<size<<" c:"<<connect <<endl;

  edge = new int[size*connect];

  for(node=0;(node<size);node++)
    for(side=0;(side<connect);side++)
      {
	while (fp.peek() == '#')
	  {
	    fp.get(com_buf,MAXLINE,'\n');
	    if (DEBUG > 50)
	      cout << "Disregard: " << com_buf << endl;
	    fp.get(c);
	  }
	fp >> edge[(node*connect)+side];
	fp.get(c);
      }
}

// Information output (primarily for debugging)
void Problem::Show(int debug_level)
{
  if (debug_level > 0)
    {
      int node,side;
      cout << "\nS:"<<size<<" C:"<<connect;
      for(node=0;(node<size);node++)
	for(side=0,cout<<endl;(side<connect);side++)
	  cout << edge[(node*connect)+side] << " ";
      cout << endl;
    }
}

// default Show
void Problem::Show(void)
{
  Show(0);
}
